@@ -7,7 +7,6 @@ import (
 
 	"gitlab.com/gitlab-org/gitlab-runner/executors/docker/internal/volumes"
 	docker_helpers "gitlab.com/gitlab-org/gitlab-runner/helpers/docker"
-	"gitlab.com/gitlab-org/gitlab-runner/helpers/featureflags"
 )
 
 type volumesManagerAdapter struct {
@@ -28,10 +27,6 @@ func (a *volumesManagerAdapter) RemoveContainer(ctx context.Context, id string)
 	return a.e.removeContainer(ctx, id)
 }
 
-func (e *executor) checkOutdatedHelperImage() bool {
-	return !e.Build.IsFeatureFlagOn(featureflags.DockerHelperImageV2) && e.Config.Docker.HelperImage != ""
-}
-
 var createVolumesManager = func(e *executor) (volumes.Manager, error) {
 	adapter := &volumesManagerAdapter{
 		Client: e.client,
@@ -48,7 +43,6 @@ var createVolumesManager = func(e *executor) (volumes.Manager, error) {
 		&e.BuildLogger,
 		adapter,
 		helperImage,
-		e.checkOutdatedHelperImage(),
 	)
 
 	config := volumes.ManagerConfig{
