@@ -277,7 +277,7 @@ func (n *GitLabClient) RequestJob(config common.RunnerConfig, sessionInfo *commo
 	switch result {
 	case http.StatusCreated:
 		config.Log().WithFields(logrus.Fields{
-			"job":      strconv.Itoa(response.ID),
+			"job":      response.ID,
 			"repo_url": response.RepoCleanURL(),
 		}).Println("Checking for jobs...", "received")
 		addTLSData(&response, tlsData)
@@ -530,10 +530,14 @@ func (n *GitLabClient) DownloadArtifacts(config common.JobCredentials, artifacts
 	}
 }
 
-func (n *GitLabClient) ProcessJob(config common.RunnerConfig, jobCredentials *common.JobCredentials) common.JobTrace {
-	trace := newJobTrace(n, config, jobCredentials)
+func (n *GitLabClient) ProcessJob(config common.RunnerConfig, jobCredentials *common.JobCredentials) (common.JobTrace, error) {
+	trace, err := newJobTrace(n, config, jobCredentials)
+	if err != nil {
+		return nil, err
+	}
+
 	trace.start()
-	return trace
+	return trace, nil
 }
 
 func NewGitLabClientWithRequestStatusesMap(rsMap *APIRequestStatusesMap) *GitLabClient {
